`timescale 1ps / 1ps
//*****************************************************************************
// File Name            : stepper_motor_uni.v
//-----------------------------------------------------------------------------
// Function             : unipolar stepper motor controller
//                        
//-----------------------------------------------------------------------------
// Designer             : yokomizo 
//-----------------------------------------------------------------------------
// History
// -.-- 2014/9/05@a
//*****************************************************************************

module stepper_motor_uni (
  clk,
  rstb,
  start,
  en,
  cw,
  step_t,
  step_n,
  busy,
  m_a,
  m_a_b,
  m_b,
  m_b_b
);

  input clk;
  input rstb;
  input start;
  input en;
  input cw;
  input [3:0] step_t;
  input [7:0] step_n;
  output busy;
  output m_a;
  output m_a_b;
  output m_b;
  output m_b_b;
  
  reg start_d1;
  reg start_d2;
  wire    start_sig;
  reg cnt_en;
  reg [3:0]m_out;
  reg [15:0] cnt_ms_base;
  reg [3:0] cnt_step_base;
  reg [7:0] cnt_step;
  reg [7:0] cnt_step_stop;
  wire    step_en;
  wire    ms_sig;

parameter p_1ms_cnt = 49999 ;//CLK=50MHẑƂ1ms̃JEgl
	  
//M --------------------------------------------------------------------------------- 
//start̗茟o   
always @ (posedge clk or negedge rstb )
if (rstb==1'b0)
  begin 
    start_d1 <= 1'b0;
    start_d2 <= 1'b0;
  end
else
  begin
    start_d1 <= start;
    start_d2 <= start_d1;
  end
  
 assign start_sig = ((start_d1==1'b0)&& ( start_d2==1'b1))?1'b1:1'b0;
 
//~Xebvʒu̎Zo 
always @ (posedge clk or negedge rstb )
if (rstb==1'b0) 
  cnt_step_stop <= 8'h0;
else
  if(start_sig==1'b1)
    if(cw==1'b1)
      cnt_step_stop <= cnt_step + step_n;
    else
      cnt_step_stop <= cnt_step - step_n;
	  
//Cl[u   
always @ (posedge clk or negedge rstb )
if (rstb==1'b0) 
  cnt_en <= 1'b0;
else
  if(en==1'b1)
    //JEgJn
    if (start_sig==1'b1)
       cnt_en <= 1'b1;
	else
	  //ݒ萔܂ŉ]fBZ[u
	  if (cnt_step==cnt_step_stop)
        cnt_en <= 1'b0;
	  else
	    cnt_en <= cnt_en;
  else
    cnt_en <= 1'b0;
	
assign busy = cnt_en;
//XebvJE^ --------------------------------------------------------------------------------- 
//1ms߂JE^
always @ (posedge clk or negedge rstb )
if (rstb==1'b0) 
  cnt_ms_base <= 16'd0;
else
  if(cnt_en==1'b1)
    if ((ms_sig == 1'd1)||(start_sig==1'b1))
      cnt_ms_base <= 16'd0;
    else
      cnt_ms_base <= cnt_ms_base + 16'd1;
  else
      cnt_ms_base <= cnt_ms_base ;
	  
//1ms1clk1ɂȂ
assign  ms_sig =((cnt_ms_base == p_1ms_cnt)&&(cnt_en==1'b1))?1'b1:1'b0; 

//XebvJEg
always @ (posedge clk or negedge rstb )
if (rstb==1'b0) 
  cnt_step_base <= 4'd0;
else
  if(cnt_en==1'b1)
    if(start_sig==1'b1)
      cnt_step_base <= 4'd0;
    else if(ms_sig == 1'b1)
      //cstep_t̒lŎύX
      if (cnt_step_base >= step_t)
        cnt_step_base <= 4'd0;
      else
        cnt_step_base <= cnt_step_base + 4'd1;
  else  
    cnt_step_base <= cnt_step_base ;
	  
//XebvECl[u@1Xebv1clk1ɂȂ
assign step_en =  ((cnt_step_base == step_t)&&(cnt_en == 1'b1)&&(ms_sig == 1'b1))?1'b1:1'b0;
   
//XebvJE^ ---------------------------------------------------------------------------------
//XebṽJEg  
always @ (posedge clk or negedge rstb )
if (rstb==1'b0) 
  cnt_step <= 8'h00;
else
    if (step_en == 1'b1)
      if(cw==1'b1)  //CW
        if (cnt_step == 8'hff)
          cnt_step <= 8'h0;
        else
          cnt_step <= cnt_step + 8'd1;
      else          //CCW
        if (cnt_step == 8'h0)
          cnt_step <= 8'hff;
        else
          cnt_step <= cnt_step - 8'd1;
    else  
      cnt_step <= cnt_step ;

//[^pX
always @ (posedge clk or negedge rstb )
if (rstb==1'b0) 
  m_out <= 4'b1001;
else
   case (cnt_step[1:0])
      2'd0:m_out <= 4'b1001;
      2'd1:m_out <= 4'b0101;
      2'd2:m_out <= 4'b0110;
      2'd3:m_out <= 4'b1010;
    endcase // case(cnt_m_out)
	
assign m_a   = m_out[0];
assign m_a_b = m_out[1];
assign m_b   = m_out[2];   
assign m_b_b = m_out[3];
      
endmodule






